<?php

namespace EventPortal\Domain\Team\Models;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Table\Concerns\HasTable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Sluggable\HasSlug;
use Spatie\Sluggable\SlugOptions;

class Team extends Model implements AuditableInterface
{
    use HasTable, Auditable, HasSlug;

    public const KEY = 'team';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'slug',
        'description',
    ];

    public function getSlugOptions(): SlugOptions
    {
        return SlugOptions::create()
            ->generateSlugsFrom('name')
            ->saveSlugsTo('slug');
    }

    public function admins(): MorphToMany
    {
        return $this->morphedByMany(Admin::class, 'teamable')->withTimestamps();
    }

    public function events(): MorphToMany
    {
        return $this->morphedByMany(Event::class, 'teamable')->withTimestamps();
    }

    public static function asOptions(): array
    {
        return Team::query()
            ->when(Auth::check(), function ($query) {
                return $query->forAdminInTeam();
            })
            ->get()
            ->map(function (Team $team) {
                return Option::make($team->name)->value($team->id);
            })
            ->toArray();
    }

    protected static function booted(): void
    {
        static::creating(static function (Team $team): void {
            /**
             * If no slug is set, create one from the name. (I'm guessing this
             * will be replaced at some point with a sluggable package)
             */
            $team->slug = $team->slug ?? Str::slug($team->name);
        });
    }

    public function scopeForAdminInTeam(Builder $query): Builder
    {
        $admin = Auth::user()->admin;

        if ($admin->hasRole('super-admin')) {
            return $query;
        }

        return $query->whereIn('teams.id', $admin->teams->pluck('id'));
    }
}
