<?php

namespace EventPortal\Domain\Util;

use Closure;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;
use Illuminate\Support\Traits\Macroable;
use Symfony\Component\VarDumper\VarDumper;

class Stringable
{
    use Macroable;

    protected $value;

    /**
     * @param mixed $value
     */
    public function __construct($value = '')
    {
        $this->value = (string) $value;
    }

    public function after(string $search): self
    {
        return new static(Str::after($this->value, $search));
    }

    public function afterLast(string $search): self
    {
        return new static(Str::afterLast($this->value, $search));
    }

    /**
     * @param mixed ...$values
     */
    public function append(...$values): self
    {
        return new static($this->value.implode('', $values));
    }

    public function ascii(string $language = 'en'): self
    {
        return new static(Str::ascii($this->value, $language));
    }

    public function basename(string $suffix = ''): self
    {
        return new static(basename($this->value, $suffix));
    }

    public function before(string $search): self
    {
        return new static(Str::before($this->value, $search));
    }

    public function beforeLast(string $search): self
    {
        return new static(Str::beforeLast($this->value, $search));
    }

    public function camel(): self
    {
        return new static(Str::camel($this->value));
    }

    /**
     * @param  string|array  $needles
     */
    public function contains($needles): bool
    {
        return Str::contains($this->value, $needles);
    }

    public function containsAll(array $needles): bool
    {
        return Str::containsAll($this->value, $needles);
    }

    public function dirname(int $levels = 1): self
    {
        return new static(dirname($this->value, $levels));
    }

    /**
     * @param  string|array  $needles
     */
    public function endsWith($needles): bool
    {
        return Str::endsWith($this->value, $needles);
    }

    public function exactly(string $value): bool
    {
        return $this->value === $value;
    }

    public function explode(string $delimiter, int $limit = PHP_INT_MAX): Collection
    {
        return collect(explode($delimiter, $this->value, $limit));
    }

    /**
     * @param  string|int  $pattern
     */
    public function split($pattern, int $limit = -1, int $flags = 0): Collection
    {
        if (filter_var($pattern, FILTER_VALIDATE_INT) !== false) {
            return collect(mb_str_split($this->value, $pattern));
        }

        $segments = preg_split($pattern, $this->value, $limit, $flags);

        /** phpcs:disable */
        return ! empty($segments) ? collect($segments) : collect();
        /** phpcs:enable */
    }

    public function finish(string $cap): self
    {
        return new static(Str::finish($this->value, $cap));
    }

    /**
     * @param  string|array  $pattern
     */
    public function is($pattern): bool
    {
        return Str::is($pattern, $this->value);
    }

    public function isEmpty(): bool
    {
        return $this->value === '';
    }

    public function isNotEmpty(): bool
    {
        return ! $this->isEmpty();
    }

    public function kebab(): self
    {
        return new static(Str::kebab($this->value));
    }

    public function length(?string $encoding = null): int
    {
        return Str::length($this->value, $encoding);
    }

    public function limit(int $limit = 100, string $end = '...'): self
    {
        return new static(Str::limit($this->value, $limit, $end));
    }

    public function lower(): self
    {
        return new static(Str::lower($this->value));
    }

    public function match(string $pattern): ?self
    {
        preg_match($pattern, $this->value, $matches);

        if (! $matches) {
            return new static;
        }

        return new static($matches[1] ?? $matches[0]);
    }

    public function matchAll(string $pattern): Collection
    {
        preg_match_all($pattern, $this->value, $matches);

        /** phpcs:disable */
        if (empty($matches[0])) {
            /** phpcs:enable */
            return collect();
        }

        return collect($matches[1] ?? $matches[0]);
    }

    public function parseCallback(?string $default = null): array
    {
        return Str::parseCallback($this->value, $default);
    }

    public function plural(int $count = 2): self
    {
        return new static(Str::plural($this->value, $count));
    }

    public function pluralStudly(int $count = 2): self
    {
        return new static(Str::pluralStudly($this->value, $count));
    }

    /**
     * @param mixed $values
     */
    public function prepend(...$values): self
    {
        return new static(implode('', $values).$this->value);
    }

    /**
     * @param string|string[] $search
     * @param string|string[] $replace
     */
    public function replace($search, $replace): self
    {
        return new static(str_replace($search, $replace, $this->value));
    }

    public function replaceArray(string $search, array $replace): self
    {
        return new static(Str::replaceArray($search, $replace, $this->value));
    }

    public function replaceFirst(string $search, string $replace): self
    {
        return new static(Str::replaceFirst($search, $replace, $this->value));
    }

    public function replaceLast(string $search, string $replace): self
    {
        return new static(Str::replaceLast($search, $replace, $this->value));
    }

    /**
     * @param \Closure|string  $replace
     */
    public function replaceMatches(string $pattern, $replace, int $limit = -1): self
    {
        if ($replace instanceof Closure) {
            return new static(preg_replace_callback($pattern, $replace, $this->value, $limit));
        }

        return new static(preg_replace($pattern, $replace, $this->value, $limit));
    }

    public function start(string $prefix): self
    {
        return new static(Str::start($this->value, $prefix));
    }

    /**
     * Convert the given string to upper-case.
     *
     * @return static
     */
    public function upper()
    {
        return new static(Str::upper($this->value));
    }

    /**
     * Convert the given string to title case.
     *
     * @return static
     */
    public function title()
    {
        return new static(Str::title($this->value));
    }

    /**
     * Get the singular form of an English word.
     *
     * @return static
     */
    public function singular()
    {
        return new static(Str::singular($this->value));
    }

    public function slug(string $separator = '-', ?string $language = 'en'): self
    {
        return new static(Str::slug($this->value, $separator, $language));
    }

    public function snake(string $delimiter = '_'): self
    {
        return new static(Str::snake($this->value, $delimiter));
    }

    /**
     * @param  string|array  $needles
     */
    public function startsWith($needles): bool
    {
        return Str::startsWith($this->value, $needles);
    }

    public function studly(): self
    {
        return new static(Str::studly($this->value));
    }

    public function substr(int $start, ?int $length = null): self
    {
        return new static(Str::substr($this->value, $start, $length));
    }

    public function trim(): self
    {
        return new static(trim(...array_merge([$this->value], func_get_args())));
    }

    public function ltrim(): self
    {
        return new static(ltrim(...array_merge([$this->value], func_get_args())));
    }

    public function rtrim(): self
    {
        return new static(rtrim(...array_merge([$this->value], func_get_args())));
    }

    public function ucfirst(): self
    {
        return new static(Str::ucfirst($this->value));
    }

    /**
     * @param  mixed  $value
     * @return mixed|$this
     */
    public function when($value, callable $callback, ?callable $default = null)
    {
        if ($value) {
            return $callback($this, $value) ?: $this;
        }

        if ($default) {
            return $default($this, $value) ?: $this;
        }

        return $this;
    }

    public function whenEmpty(callable $callback): self
    {
        if ($this->isEmpty()) {
            $result = $callback($this);

            return is_null($result) ? $this : $result;
        }

        return $this;
    }

    public function words(int $words = 100, string $end = '...'): self
    {
        return new static(Str::words($this->value, $words, $end));
    }

    public function dump(): self
    {
        VarDumper::dump($this->value);

        return $this;
    }

    public function dd(): void
    {
        $this->dump();

        exit(1);
    }

    /**
     * @return mixed
     */
    public function __get(string $key)
    {
        return $this->{$key}();
    }

    public function __toString(): string
    {
        return (string) $this->value;
    }
}
