<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TeamStoreTest extends TestCase
{
    use DatabaseMigrations, SeedRolesAndPermissions;

    /** @test */
    public function a_team_can_be_created()
    {
        $admin = factory(Admin::class)->state('admin')->create();

        $this
            ->actingAsAdmin($admin)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'description' => 'New team description',
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Team',
            'description' => 'New team description',
        ]);
    }

    /** @test **/
    public function the_team_name_is_required()
    {
        $admin = factory(Admin::class)->state('admin')->create();

        $this
            ->actingAsAdmin($admin)
            ->post(route('admin.team.store'), [
                'name' => null,
                'description' => 'No name',
            ])
            ->assertSessionHasErrors([
                'name',
            ]);

        $this->assertDatabaseMissing('teams', [
            'description' => 'No name',
        ]);
    }

    /** @test **/
    public function the_slug_can_be_generated_automatically()
    {
        $this->withoutExceptionHandling();
        $admin = factory(Admin::class)->state('admin')->create();

        $this
            ->actingAsAdmin($admin)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'slug' => null,
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Team',
            'slug' => 'new-team'
        ]);
    }

    /** @test **/
    public function members_can_be_added_when_creating_a_team()
    {
        $authAdmin = factory(Admin::class)->create();

        [$teamMember1, $teamMember2] = factory(Admin::class, 2)->create();

        $this
            ->actingAsSuperAdmin($authAdmin)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'members' => [
                    $teamMember1->id,
                    $teamMember2->id,
                ],
            ]);

        $team = Team::first();
        $teamUserIds = $team->admins->pluck('id');

        $this->assertCount(2, $team->admins);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertContains($teamMember2->id, $teamUserIds);
        $this->assertNotContains($authAdmin->id, $teamUserIds);
    }

    /** @test **/
    public function only_valid_members_can_be_added()
    {
        $this->markTestSkipped('Need to complete');

        $this
            ->actingAsSuperAdmin()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'members' => [
                    321,
                    123,
                ],
            ])
            ->assertSessionHasErrors([
                'members',
            ]);

        $this->assertDatabaseMissing('teams', [
            'name' => 'New Team',
        ]);
    }
}
